@@ -30,15 +30,11 @@ from ._logging import (
     setup_http_handler,
 )
 from ._reporting import EXPERIMENT_CREATED, EXPERIMENT_CREATION_FAILED
+from ._typing import Any, Dict, Optional
 from .api import API, APIExperiment
 from .comet import Streamer, format_url, generate_guid, is_valid_experiment_key
-from .config import (
-    get_api_key,
-    get_config,
-    get_global_experiment,
-    get_previous_experiment,
-    get_ws_url,
-)
+from .config import get_global_experiment  # noqa
+from .config import get_api_key, get_config, get_previous_experiment, get_ws_url
 from .connection import (
     INITIAL_BEAT_DURATION,
     RestServerConnection,
@@ -50,6 +46,7 @@ from .connection import (
 from .exceptions import BadCallbackArguments, InvalidAPIKey
 from .experiment import BaseExperiment
 from .feature_toggles import HTTP_LOGGING, FeatureToggles
+from .json_encoder import NestedEncoder
 from .loggers.fastai_logger import patch as fastai_patch
 from .loggers.keras_logger import patch as keras_patch
 from .loggers.mlflow_logger import patch as mlflow_patch
@@ -57,6 +54,8 @@ from .loggers.pytorch_logger import patch as pytorch_patch
 from .loggers.sklearn_logger import patch as sklearn_patch
 from .loggers.tensorboard_logger import patch as tb_patch
 from .loggers.tensorflow_logger import patch as tf_patch
+from .loggers.xgboost_logger import patch as xg_patch
+from .logging_messages import ONLINE_EXPERIMENT_THROTTLED
 from .monkey_patching import CometModuleFinder
 from .offline import OfflineExperiment
 from .optimizer import Optimizer
@@ -87,6 +86,7 @@ if not get_config("comet.disable_auto_logging"):
     pytorch_patch(MODULE_FINDER)
     fastai_patch(MODULE_FINDER)
     mlflow_patch(MODULE_FINDER)
+    xg_patch(MODULE_FINDER)
     MODULE_FINDER.start()
 
 # Configure the logging
@@ -131,6 +131,7 @@ class Experiment(BaseExperiment):
         log_env_host=True,
         display_summary=None,
         log_env_cpu=True,
+        display_summary_level=1,
     ):
         """
         Creates a new experiment on the Comet.ml frontend.
@@ -168,13 +169,13 @@ class Experiment(BaseExperiment):
                 automatic collection of git details
             log_git_patch: Default(True) - allow you to enable/disable the
                 automatic collection of git patch
-            display_summary: Default(True) - control whether the summary is
-                displayed on the console or not. If disabled, the summary
-                notification is still sent.
+            display_summary_level: Default(1) - control the summary detail that is
+                displayed on the console at end of experiment. If 0, the summary
+                notification is still sent. Valid values are 0 to 2.
             disabled: Default(False) - allows you to disable all network
                 communication with the Comet.ml backend. It is useful when you
-                just needs to works on your machine-learning scripts and need
-                to relaunch them several times at a time.
+                want to test to make sure everything is working, without actually
+                logging anything.
         """
         self.config = get_config()
 
@@ -202,12 +203,12 @@ class Experiment(BaseExperiment):
             disabled=disabled,
             log_env_gpu=log_env_gpu,
             log_env_host=log_env_host,
-            display_summary=display_summary,
+            display_summary_level=display_summary_level,
             log_env_cpu=log_env_cpu,
         )
 
         self.ws_connection = None
-        self.connection = None
+        self.connection = None  # type: Optional[RestServerConnection]
         self.rest_api_client = None
 
         if disabled is not True:
@@ -235,7 +236,9 @@ class Experiment(BaseExperiment):
 
         server_address = get_backend_address()
 
-        self.connection = RestServerConnection(self.api_key, self.id, server_address)
+        self.connection = RestServerConnection(
+            self.api_key, self.id, server_address, self.config["comet.timeout.http"]
+        )
         self.rest_api_client = get_rest_api_client("v2", api_key=self.api_key)
 
         try:
@@ -369,6 +372,15 @@ class Experiment(BaseExperiment):
             if self.connection is not None:
                 self.connection.close()
 
+        # Display throttling message
+        try:
+            experiment_metadata = self.rest_api_client.get_experiment_metadata(self.id)
+
+            if experiment_metadata.get("throttle", False):
+                LOGGER.warning(ONLINE_EXPERIMENT_THROTTLED)
+        except Exception:
+            LOGGER.debug("Failed to check experiment metadata", exc_info=True)
+
         return successful_clean
 
     @property
@@ -434,47 +446,6 @@ class Experiment(BaseExperiment):
         except Exception:
             LOGGER.warning(ADD_SYMLINK_ERROR, project_name, exc_info=True)
 
-    def display(self, clear=False, wait=True, new=0, autoraise=True, tab=None):
-        """
-        Show the comet.ml experiment page in an IFrame in a
-        Jupyter notebook or Jupyter lab, OR open a browser
-        window or tab.
-
-        For Jupyter environments:
-
-        Args:
-            clear: to clear the output area, use clear=True
-            wait: to wait for the next displayed item, use
-                  wait=True (cuts down on flashing)
-
-        For non-Jupyter environments:
-
-        Args:
-            new: open a new browser window if new=1, otherwise re-use
-                 existing window/tab
-            autoraise: make the browser tab/window active
-        """
-        if self._in_jupyter_environment():
-            from IPython.display import display, IFrame, clear_output
-
-            if clear:
-                clear_output(wait=wait)
-            display(
-                IFrame(
-                    src=get_global_experiment()._get_experiment_url(tab),
-                    width="100%",
-                    height="800px",
-                )
-            )
-        else:
-            import webbrowser
-
-            webbrowser.open(
-                get_global_experiment()._get_experiment_url(tab),
-                new=new,
-                autoraise=autoraise,
-            )
-
     def add_tag(self, tag):
         """
         Add a tag to the experiment. Tags will be shown in the dashboard.
@@ -534,6 +505,7 @@ class Experiment(BaseExperiment):
         self.connection.register_rpc(remote_action_definition)
 
     def send_notification(self, title, status=None, additional_data=None):
+        # type: (str, Optional[str], Optional[Dict[str, Any]]) -> None
         """
         Send yourself a notification through email when an experiment
         ends.
@@ -561,7 +533,12 @@ class Experiment(BaseExperiment):
                 additional_data = {}
 
             self.connection.send_notification(
-                title, status, name, self._get_experiment_url(), additional_data
+                title,
+                status,
+                name,
+                self._get_experiment_url(),
+                additional_data,
+                custom_encoder=NestedEncoder,
             )
 
         except Exception:
@@ -618,36 +595,65 @@ class ExistingExperiment(Experiment):
             project_name: Optional. Send your experiment to a specific project. Otherwise will be sent to `Uncategorized Experiments`.
                              If project name does not already exists Comet.ml will create a new project.
             workspace: Optional. Attach an experiment to a project that belongs to this workspace
-            log_code: Default(True) - allows you to enable/disable code logging
-            log_graph: Default(True) - allows you to enable/disable automatic computation graph logging.
+            log_code: Default(False) - allows you to enable/disable code logging
+            log_graph: Default(False) - allows you to enable/disable automatic computation graph logging.
             auto_param_logging: Default(True) - allows you to enable/disable hyper parameters logging
             auto_metric_logging: Default(True) - allows you to enable/disable metrics logging
-            parse_args: Default(True) - allows you to enable/disable automatic parsing of CLI arguments
-            auto_output_logging: Default("native") - allows you to select
-                which output logging mode to use. The default is `"native"`
+            parse_args: Default(False) - allows you to enable/disable automatic parsing of CLI arguments
+            auto_output_logging: Default("default") - allows you to select
+                which output logging mode to use. You can pass `"native"`
                 which will log all output even when it originated from a C
                 native library. You can also pass `"simple"` which will work
                 only for output made by Python code. If you want to disable
-                automatic output logging, you can pass `None`.
-            log_env_details: Default(True) - log various environment
+                automatic output logging, you can pass `False`. The default is
+                `"default"` which will detect your environment and deactivate
+                the output logging for IPython and Jupyter environment and
+                sets `"native"` in the other cases.
+            log_env_details: Default(False) - log various environment
                 information in order to identify where the script is running
-            log_env_gpu: Default(True) - allow you to enable/disable the
+            log_env_gpu: Default(False) - allow you to enable/disable the
                 automatic collection of gpu details and metrics (utilization, memory usage etc..).
                 `log_env_details` must also be true.
-            log_env_cpu: Default(True) - allow you to enable/disable the
+            log_env_cpu: Default(False) - allow you to enable/disable the
                 automatic collection of cpu details and metrics (utilization, memory usage etc..).
                 `log_env_details` must also be true.
-            log_env_host: Default(True) - allow you to enable/disable the
+            log_env_host: Default(False) - allow you to enable/disable the
                 automatic collection of host information (ip, hostname, python version, user etc...).
                 `log_env_details` must also be true.
-            log_git_metadata: Default(True) - allow you to enable/disable the
+            log_git_metadata: Default(False) - allow you to enable/disable the
                 automatic collection of git details
-            log_git_patch: Default(True) - allow you to enable/disable the
+            log_git_patch: Default(False) - allow you to enable/disable the
                 automatic collection of git patch
+            display_summary_level: Default(1) - control the summary detail that is
+                displayed on the console at end of experiment. If 0, the summary
+                notification is still sent. Valid values are 0 to 2.
             disabled: Default(False) - allows you to disable all network
                 communication with the Comet.ml backend. It is useful when you
                 just needs to works on your machine-learning scripts and need
                 to relaunch them several times at a time.
+
+        Note: ExistingExperiment does not alter nor destroy previously
+        logged information. To override or add to previous information
+        you will have to set the appropriate following parameters to True:
+
+        * log_code
+        * log_graph
+        * parse_args
+        * log_env_details
+        * log_git_metadata
+        * log_git_patch
+        * log_env_gpu
+        * log_env_cpu
+        * log_env_host
+
+        For example, to contiue to collect GPU information in an
+        `ExistingExperiment` you will need to override these parameters:
+
+        ```python
+        >>> experiment = ExistingExperiment(
+        ...                 log_env_details=True,
+        ...                 log_env_gpu=True)
+        ```
         """
         # Validate the previous experiment id
         self.config = get_config()
