@@ -18,6 +18,7 @@ from os.path import basename, join
 from comet_ml._logging import check_module
 from comet_ml.offline import OfflineExperiment
 
+from .._typing import Optional
 from ..config import get_config, get_global_experiment, set_global_experiment
 from ..experiment import BaseExperiment
 from ..logging_messages import (
@@ -46,11 +47,11 @@ def _reset_global_state():
 
     LOG_MODEL_MODEL_NAME = None
     PROJECT_NAME = None
-    IMPLICIT_START_RUN = None
+    IMPLICIT_START_RUN = False
 
 
 def _create_experiment(experiment_name=None):
-    # type: () -> BaseExperiment
+    # type: (Optional[str]) -> BaseExperiment
     LOGGER.debug("Creating new Experiment for MLFlow, implicit? %r", IMPLICIT_START_RUN)
 
     global PROJECT_NAME
@@ -91,7 +92,7 @@ def _create_experiment(experiment_name=None):
 
 
 def _get_or_create_experiment(experiment):
-    # type: () -> BaseExperiment
+    # type: (Optional[BaseExperiment]) -> BaseExperiment
     if experiment:
         return experiment
 
