@@ -142,6 +142,10 @@ LOG_DATASET_ERROR = "Failed to create dataset hash"
 
 OS_PACKAGE_MSG_SENDING_ERROR = "Error sending os_packages message, got %d %r"
 
+MODEL_GRAPH_MSG_SENDING_ERROR = "Error sending model graph message, got %d %r"
+
+CLOUD_DETAILS_MSG_SENDING_ERROR = "Error sending cloud details messages, got %d %r"
+
 OPTIMIZER_KWARGS_CONSTRUCTOR_DEPRACETED = "Passing Experiment through Optimizer constructor is deprecated; pass them to Optimizer.get_experiments or Optimizer.next"
 
 BACKEND_VERSION_CHECK_ERROR = "Failed to check backend version at URL: %r"
@@ -153,3 +157,5 @@ MLFLOW_NESTED_RUN_UNSUPPORTED = "MLFlow Nested Runs are not tracked in Comet.ml
 MLFLOW_OFFLINE_EXPERIMENT_FALLBACK = "No Comet API Key was found, creating an OfflineExperiment. Set up your API Key to get the full Comet experience https://www.comet.ml/docs/python-sdk/advanced/#python-configuration"
 
 MLFLOW_RESUMED_RUN = "Resumed MLFlow run are tracked as new Comet.ml Experiments"
+
+ONLINE_EXPERIMENT_THROTTLED = "Experiment has been throttled, some data will be missing"
