@@ -33,22 +33,12 @@ from zipfile import ZipFile
 from jsonschema import ValidationError
 from jsonschema.validators import validator_for
 
-from ._logging import (
-    OFFLINE_EXPERIMENT_ALREADY_UPLOADED,
-    OFFLINE_EXPERIMENT_END,
-    OFFLINE_EXPERIMENT_INVALID_UPLOAD_MSG,
-    OFFLINE_EXPERIMENT_INVALID_WS_MSG,
-    OFFLINE_EXPERIMENT_TEMPORARY_DIRECTORY,
-    OFFLINE_SENDER_ENDS,
-    OFFLINE_SENDER_STARTS,
-    OS_PACKAGE_MSG_SENDING_ERROR,
-)
 from ._reporting import (
     EXPERIMENT_CREATED,
     OFFLINE_INVALID_UPLOAD_MSG,
     OFFLINE_INVALID_WS_MSG,
 )
-from ._typing import List
+from ._typing import List, Optional
 from .comet import OfflineStreamer, format_url, generate_guid
 from .config import get_api_key, get_config
 from .connection import (
@@ -67,6 +57,18 @@ from .exceptions import (
 from .experiment import BaseExperiment
 from .feature_toggles import FeatureToggles
 from .file_uploader import upload_file_thread
+from .logging_messages import (
+    CLOUD_DETAILS_MSG_SENDING_ERROR,
+    MODEL_GRAPH_MSG_SENDING_ERROR,
+    OFFLINE_EXPERIMENT_ALREADY_UPLOADED,
+    OFFLINE_EXPERIMENT_END,
+    OFFLINE_EXPERIMENT_INVALID_UPLOAD_MSG,
+    OFFLINE_EXPERIMENT_INVALID_WS_MSG,
+    OFFLINE_EXPERIMENT_TEMPORARY_DIRECTORY,
+    OFFLINE_SENDER_ENDS,
+    OFFLINE_SENDER_STARTS,
+    OS_PACKAGE_MSG_SENDING_ERROR,
+)
 from .metrics import MetricsSampler
 from .utils import local_timestamp
 
@@ -76,25 +78,27 @@ LOGGER = logging.getLogger(__name__)
 class OfflineExperiment(BaseExperiment):
     def __init__(
         self,
-        project_name=None,
-        workspace=None,
-        log_code=True,
-        log_graph=True,
-        auto_param_logging=True,
-        auto_metric_logging=True,
-        parse_args=True,
-        auto_output_logging="default",
-        log_env_details=True,
-        log_git_metadata=True,
-        log_git_patch=True,
-        disabled=False,
-        offline_directory=None,
-        log_env_gpu=True,
-        log_env_host=True,
-        api_key=None,  # Optional, except for on-line operations
-        display_summary=None,
-        log_env_cpu=True,
+        project_name=None,  # type: Optional[str]
+        workspace=None,  # type: Optional[str]
+        log_code=True,  # type: Optional[bool]
+        log_graph=True,  # type: Optional[bool]
+        auto_param_logging=True,  # type: Optional[bool]
+        auto_metric_logging=True,  # type: Optional[bool]
+        parse_args=True,  # type: Optional[bool]
+        auto_output_logging="default",  # type: Optional[str]
+        log_env_details=True,  # type: Optional[bool]
+        log_git_metadata=True,  # type: Optional[bool]
+        log_git_patch=True,  # type: Optional[bool]
+        disabled=False,  # type: Optional[bool]
+        offline_directory=None,  # type: Optional[str]
+        log_env_gpu=True,  # type: Optional[bool]
+        log_env_host=True,  # type: Optional[bool]
+        api_key=None,  # type: Optional[str]
+        display_summary=None,  # type: Optional[bool]
+        log_env_cpu=True,  # type: Optional[bool]
+        display_summary_level=1,  # type: Optional[int]
     ):
+        # type: (...) -> None
         """
         Creates a new experiment on the Comet.ml frontend.
         Args:
@@ -128,14 +132,16 @@ class OfflineExperiment(BaseExperiment):
                 `log_env_details` must also be true.
             log_git_metadata: Default(True) - allow you to enable/disable the
                 automatic collection of git details
+            display_summary_level: Default(1) - control the summary detail that is
+                displayed on the console at end of experiment. If 0, the summary
+                notification is still sent. Valid values are 0 to 2.
             disabled: Default(False) - allows you to disable all network
                 communication with the Comet.ml backend. It is useful when you
-                just needs to works on your machine-learning scripts and need
-                to relaunch them several times at a time.
+                want to test to make sure everything is working, without actually
+                logging anything.
             offline_directory: the directory used to save the offline archive
                 for the experiment.
         """
-        self.tmpdir = tempfile.mkdtemp()
         self.config = get_config()
 
         self.api_key = get_api_key(
@@ -171,10 +177,9 @@ class OfflineExperiment(BaseExperiment):
             log_env_host,
             display_summary,
             log_env_cpu,
+            display_summary_level,
         )
 
-        self._force_copy_to_tmp = True
-
         if not disabled:
             # Check that the offline directory is usable
             try:
@@ -296,23 +301,9 @@ class OfflineExperiment(BaseExperiment):
         # TODO WHAT TO DO WITH REPORTING?
         pass
 
-    def _get_experiment_url(self):
+    def _get_experiment_url(self, tab=None):
         return "[OfflineExperiment will get URL after upload]"
 
-    def _upload_file_like(
-        self, file_data, upload_type, max_size, url_params, too_big_log_msg, copy_to_tmp
-    ):
-        if not copy_to_tmp:
-            msg = (
-                "Overriding copy_to_tmp to force writing in on disk for offline storage"
-            )
-            LOGGER.warning(msg)
-            copy_to_tmp = True
-
-        return super(OfflineExperiment, self)._upload_file_like(
-            file_data, upload_type, max_size, url_params, too_big_log_msg, copy_to_tmp
-        )
-
 
 def get_validator(filename, allow_additional_properties=True):
     with open(join(dirname(__file__), join("schemas", filename))) as schema_file:
@@ -340,6 +331,22 @@ def get_os_packages_msg_validator(allow_additional_properties=False):
     )
 
 
+def get_graph_msg_validator(allow_additional_properties=False):
+    return get_validator("offline-graph-msg.schema.json", allow_additional_properties)
+
+
+def get_system_details_msg_validator(allow_additional_properties=False):
+    return get_validator(
+        "offline-system-details-msg.schema.json", allow_additional_properties
+    )
+
+
+def get_cloud_details_msg_validator(allow_additional_properties=False):
+    return get_validator(
+        "offline-cloud-details-msg.schema.json", allow_additional_properties
+    )
+
+
 def get_upload_msg_validator(allow_additional_properties=True):
     return get_validator(
         "offline-file-upload-msg.schema.json", allow_additional_properties
@@ -359,7 +366,12 @@ def unzip_offline_archive(offline_archive_path):
 
 class OfflineSender(object):
     def __init__(
-        self, api_key, offline_dir, force_reupload=False, display_level="info"
+        self,
+        api_key,
+        offline_dir,
+        force_reupload=False,
+        display_level="info",
+        validation_error=False,
     ):
         self.config = get_config()
         self.api_key = api_key
@@ -372,6 +384,9 @@ class OfflineSender(object):
         self.experiment_file_validator = get_experiment_file_validator()
         self.ws_msg_validator = get_ws_msg_validator()
         self.os_packages_msg_validator = get_os_packages_msg_validator()
+        self.graph_msg_validator = get_graph_msg_validator()
+        self.system_details_msg_validator = get_system_details_msg_validator()
+        self.cloud_details_msg_validator = get_cloud_details_msg_validator()
         self.upload_msg_validator = get_upload_msg_validator()
 
         self.server_address = get_backend_address()
@@ -379,13 +394,18 @@ class OfflineSender(object):
         self._read_experiment_file()
 
         self.connection = RestServerConnection(
-            self.api_key, self.experiment_id, self.server_address
+            self.api_key,
+            self.experiment_id,
+            self.server_address,
+            self.config["comet.timeout.http"],
         )
         self.ws_connection = None
         self.rest_api_client = None
         self.focus_link = None
         self.upload_threads = []  # type: List[threading.Thread]
 
+        self.validation_error = validation_error
+
     def send(self):
         self._handshake()
 
@@ -486,6 +506,12 @@ class OfflineSender(object):
                         self._process_upload_message(message)
                     elif message_type == "os_packages":
                         self._process_os_packages_message(message)
+                    elif message_type == "graph":
+                        self._process_graph_message(message)
+                    elif message_type == "system_details":
+                        self._process_system_details_message(message)
+                    elif message_type == "cloud_details":
+                        self._process_cloud_details_message(message)
                     else:
                         raise ValueError("Unknown message type %r", message_type)
                 except Exception:
@@ -505,6 +531,9 @@ class OfflineSender(object):
         try:
             self.ws_msg_validator.validate(message)
         except ValidationError:
+            if self.validation_error:
+                raise
+
             tb = traceback.format_exc()
             LOGGER.warning(OFFLINE_EXPERIMENT_INVALID_WS_MSG, exc_info=True)
             self.connection.report(event_name=OFFLINE_INVALID_WS_MSG, err_msg=tb)
@@ -526,6 +555,9 @@ class OfflineSender(object):
         try:
             self.upload_msg_validator.validate(message)
         except ValidationError:
+            if self.validation_error:
+                raise
+
             tb = traceback.format_exc()
             LOGGER.warning(OFFLINE_EXPERIMENT_INVALID_UPLOAD_MSG, exc_info=True)
             self.connection.report(event_name=OFFLINE_INVALID_UPLOAD_MSG, err_msg=tb)
@@ -555,6 +587,9 @@ class OfflineSender(object):
         try:
             self.os_packages_msg_validator.validate(message)
         except ValidationError:
+            if self.validation_error:
+                raise
+
             tb = traceback.format_exc()
             LOGGER.warning(OFFLINE_INVALID_WS_MSG, exc_info=True)
             self.connection.report(event_name=OFFLINE_INVALID_WS_MSG, err_msg=tb)
@@ -564,13 +599,108 @@ class OfflineSender(object):
                 self.experiment_id, message["os_packages"]
             )
         except CometRestApiException as exc:
-            LOGGER.debug(
+            LOGGER.error(
                 OS_PACKAGE_MSG_SENDING_ERROR,
                 exc.response.status_code,
                 exc.response.content,
             )
         except Exception:
-            LOGGER.debug("Error sending os_packages message", exc_info=True)
+            LOGGER.error("Error sending os_packages message", exc_info=True)
+
+    def _process_graph_message(self, message):
+        message = message["payload"]
+
+        try:
+            self.graph_msg_validator.validate(message)
+        except ValidationError:
+            if self.validation_error:
+                raise
+
+            tb = traceback.format_exc()
+            LOGGER.warning(OFFLINE_INVALID_WS_MSG, exc_info=True)
+            self.connection.report(event_name=OFFLINE_INVALID_WS_MSG, err_msg=tb)
+
+        try:
+            self.rest_api_client.set_experiment_model_graph(
+                self.experiment_id, message["graph"]
+            )
+        except CometRestApiException as exc:
+            LOGGER.error(
+                MODEL_GRAPH_MSG_SENDING_ERROR,
+                exc.response.status_code,
+                exc.response.content,
+            )
+        except Exception:
+            LOGGER.error("Error sending os_packages message", exc_info=True)
+
+    def _process_system_details_message(self, message):
+        message = message["payload"]
+
+        try:
+            self.system_details_msg_validator.validate(message)
+        except ValidationError:
+            if self.validation_error:
+                raise
+
+            tb = traceback.format_exc()
+            LOGGER.warning(OFFLINE_INVALID_WS_MSG, exc_info=True)
+            self.connection.report(event_name=OFFLINE_INVALID_WS_MSG, err_msg=tb)
+
+        try:
+            self.rest_api_client.set_experiment_system_details(
+                _os=message["os"],
+                command=message["command"],
+                env=message["env"],
+                experiment_key=self.experiment_id,
+                hostname=message["hostname"],
+                ip=message["ip"],
+                machine=message["machine"],
+                network_interfaces_ips=message["network_interfaces_ips"],
+                os_release=message["os_release"],
+                os_type=message["os_type"],
+                pid=message["pid"],
+                processor=message["processor"],
+                python_exe=message["python_exe"],
+                python_version_verbose=message["python_version_verbose"],
+                python_version=message["python_version"],
+                user=message["user"],
+            )
+        except CometRestApiException as exc:
+            LOGGER.error(
+                MODEL_GRAPH_MSG_SENDING_ERROR,
+                exc.response.status_code,
+                exc.response.content,
+            )
+        except Exception:
+            LOGGER.error("Error sending os_packages message", exc_info=True)
+
+    def _process_cloud_details_message(self, message):
+        message = message["payload"]
+
+        try:
+            self.cloud_details_msg_validator.validate(message)
+        except ValidationError:
+            if self.validation_error:
+                raise
+
+            tb = traceback.format_exc()
+            LOGGER.warning(OFFLINE_INVALID_WS_MSG, exc_info=True)
+            self.connection.report(event_name=OFFLINE_INVALID_WS_MSG, err_msg=tb)
+
+        try:
+            self.rest_api_client.set_experiment_cloud_details(
+                experiment_key=self.experiment_id,
+                provider=message["provider"],
+                cloud_metadata=message["cloud_metadata"],
+            )
+        except CometRestApiException as exc:
+            LOGGER.error(
+                CLOUD_DETAILS_MSG_SENDING_ERROR,
+                exc.response.status_code,
+                exc.response.content,
+            )
+        except Exception:
+            LOGGER.error("Error sending cloud details message", exc_info=True)
 
     def _status_report_start(self):
         self.connection.update_experiment_status(
