@@ -300,7 +300,6 @@ class Optimizer(object):
         you would not call this manually, as it is called directly
         when the experiment ends.
         """
-        metrics = experiment._summary["metric"]
         pid = experiment.optimizer["pid"]
         trial = experiment.optimizer["trial"]
         count = experiment.optimizer["count"]
@@ -310,15 +309,17 @@ class Optimizer(object):
         version = status["version"]
         score = None
         ## Find correct, min/max metric
-        if metric in metrics and len(metrics[metric]) == 2:
+        spec_metric = experiment._summary.get("metrics", metric)
+        if spec_metric:
             if "objective" not in status["spec"]:
                 # Algorithm doesn't actually need score
                 # but we send min anyway:
-                score = metrics[metric][0]
+                score = spec_metric["min"]
             elif status["spec"]["objective"] == "minimize":
-                score = metrics[metric][0]
+                score = spec_metric["min"]
             else:
-                score = metrics[metric][1]
+                score = spec_metric["max"]
+                # Set again, with whatever is in spec:
                 objective = status["spec"]["objective"]
             result = "completed"
         else:
@@ -338,7 +339,7 @@ class Optimizer(object):
         experiment.log_other(
             "optimizer_parameters",
             json.dumps(
-                {key: experiment.get_parameter(key) for key in status["parameters"]}
+                {key: experiment.params.get(key, None) for key in status["parameters"]}
             ),
         )
         if self.predictor_spec is not None:
