@@ -96,6 +96,7 @@ import time
 
 import comet_ml.bootstrap
 from comet_ml import Optimizer
+from comet_ml._typing import List
 from comet_ml.config import get_config
 
 ADDITIONAL_ARGS = True
@@ -201,7 +202,7 @@ def optimize(parsed, subcommand_args):
         environ["PYTHONPATH"] = bootstrap_dir
 
     command_line = [COMET_EXECUTABLE, parsed.PYTHON_SCRIPT] + subcommand_args
-    subprocesses = []
+    subprocesses = []  # type: List[subprocess.Popen]
 
     for j in range(parsed.parallel):
         environ["COMET_OPTIMIZER_PROCESS_ID"] = str(j)
