@@ -62,6 +62,37 @@ except ImportError:
     HAS_NUMPY = False
 
 
+INFINITY = float("inf")
+
+
+def fix_special_floats(value, _inf=INFINITY, _neginf=-INFINITY):
+    """ Fix out of bounds floats (like infinity and -infinity) and Not A
+    Number.
+    Returns either a fixed value that could be JSON encoded or the original
+    value.
+    """
+
+    try:
+        if hasattr(value, "numpy"):
+            value = value.numpy()
+
+        # Check if the value is Nan, equivalent of math.isnan
+        if math.isnan(value):
+            return "NaN"
+
+        elif value == _inf:
+            return "Infinity"
+
+        elif value == _neginf:
+            return "-Infinity"
+
+    except Exception:
+        # Value cannot be compared
+        return value
+
+    return value
+
+
 def log_once_at_level(logging_level, message, *args, **kwargs):
     """
     Log the given message once at the given level then at the DEBUG
@@ -146,22 +177,21 @@ def wait_for_empty(check_function, timeout, verbose=False, sleep_time=1):
             time.sleep(sleep_time / 20)
 
 
-def read_unix_packages():
-    # type: () -> Optional[List[str]]
-    package_status_file = "/var/lib/dpkg/status"
+def read_unix_packages(package_status_file="/var/lib/dpkg/status"):
+    # type: (str) -> Optional[List[str]]
     if os.path.isfile(package_status_file):
-        with open(package_status_file, "r") as f:
-            status = f.read()
         package = None
         os_packages = []
-        for line in status.split("\n"):
-            if line.startswith("Package: "):
-                package = line[9:]
-            if line.startswith("Version: "):
-                version = line[9:]
-                if package is not None:
-                    os_packages.append((package, version))
-                package = None
+        with open(package_status_file, "rb") as fp:
+            for binary_line in fp:
+                line = binary_line.decode("utf-8", errors="ignore").strip()
+                if line.startswith("Package: "):
+                    package = line[9:]
+                if line.startswith("Version: "):
+                    version = line[9:]
+                    if package is not None:
+                        os_packages.append((package, version))
+                    package = None
         os_packages_list = sorted(
             [("%s=%s" % (package, version)) for (package, version) in os_packages]
         )
@@ -370,8 +400,8 @@ def rescale_array(array, old_range, new_range, dtype):
         return (new_min + (array - old_min) * new_delta / old_delta).astype(dtype)
 
 
-def write_file_like_to_tmp_file(file_like_object):
-    # type: (IO) -> str
+def write_file_like_to_tmp_file(file_like_object, tmpdir):
+    # type: (IO, str) -> str
     # Copy of `shutil.copyfileobj` with binary / text detection
 
     buf = file_like_object.read(1)
@@ -382,7 +412,7 @@ def write_file_like_to_tmp_file(file_like_object):
     else:
         tmp_file_mode = "w+"
 
-    tmp_file = tempfile.NamedTemporaryFile(mode=tmp_file_mode, delete=False)
+    tmp_file = tempfile.NamedTemporaryFile(mode=tmp_file_mode, dir=tmpdir, delete=False)
     tmp_file.write(buf)
 
     # Main copy loop
@@ -416,6 +446,24 @@ def data_to_fp(data):
     return fp
 
 
+def format_bytes(size):
+    # type: (int) -> str
+    """
+    Given a size in bytes, return a sort string representation.
+    """
+    kbytes = size // 1024
+    mbytes = kbytes // 1024
+    gbytes = mbytes // 1024
+    if gbytes:
+        return "%s %s" % (gbytes, "GB")
+    elif mbytes:
+        return "%s %s" % (mbytes, "MB")
+    elif kbytes:
+        return "%s %s" % (kbytes, "KB")
+    else:
+        return "%s %s" % (size, "bytes")
+
+
 class ConfusionMatrix(object):
     """
     Data structure for holding a confusion matrix of values and their
@@ -1011,6 +1059,23 @@ class Histogram(object):
         self.values = tuple(self.create_bin_values())
         self.clear()
 
+    def __sub__(self, other):
+        if (
+            isinstance(other, Histogram)
+            and (self.start == other.start)
+            and (self.stop == other.stop)
+            and (self.step == other.step)
+            and (self.offset == other.offset)
+        ):
+
+            histogram = Histogram(self.start, self.stop, self.step, self.offset)
+            histogram.counts = [
+                abs(c2 - c1) for c1, c2 in zip(self.counts, other.counts)
+            ]
+            return histogram
+        else:
+            raise TypeError("Can't subtract Histograms of different regions")
+
     def clear(self):
         """
         Clear the counts, initializes back to zeros.
@@ -1061,8 +1126,7 @@ class Histogram(object):
         # Find initial bin:
         bucket = self.get_bin_index(values[0])
 
-        for i in range(len(values)):
-            value = values[i]
+        for i, value in enumerate(values):
             skip_count = 0
             while not (self.values[bucket] <= value < self.values[bucket + 1]):
                 skip_count += 1
@@ -1086,6 +1150,78 @@ class Histogram(object):
             [i, self.counts[i]] for i in range(len(self.counts)) if self.counts[i] > 0
         ]
 
+    @classmethod
+    def from_json(cls, data, verbose=True):
+        """
+        Given histogram JSON data, returns either a Histogram object (in
+        the case of a 2D histogram) or a list of Histogram objects (in
+        the case of a 3D histogram).
+
+        Args:
+            data: histogram filename or JSON data
+            verbose: optional, bool, if True, display the Histogram
+                after conversion
+
+        Returns: a Histogram, or list of Histograms.
+
+        Example:
+
+        ```python
+        In[1]: histogram = Histogram.from_json("histogram.json")
+
+        Histogram 3D:
+        Step: 0
+        Histogram
+        =========
+        Range Start      Range End          Count           Bins
+        -----------------------------------------------------------
+            -0.1239        -0.1003        63.7810      [506-508]
+            -0.1003        -0.0766      1006.1836      [508-511]
+            -0.0766        -0.0530      1824.0884      [511-514]
+            -0.0530        -0.0293     11803.4008      [514-521]
+            -0.0293        -0.0056     13118.4797      [521-538]
+            -0.0056         0.0180     13059.5624     [538-1023]
+             0.0180         0.0417     13133.6479    [1023-1032]
+             0.0417         0.0654      5865.2828    [1032-1037]
+             0.0654         0.0890      1616.2949    [1037-1040]
+             0.0890         0.1127       275.2784    [1040-1042]
+             0.1127         0.1363         0.0000    [1042-1044]
+        -----------------------------------------------------------
+        Total:     61766.0000
+        Out[1]:
+        <comet_ml.utils.Histogram at 0x7fac8f3819b0>
+        ```
+        """
+        if isinstance(data, str):
+            filename = os.path.expanduser(data)
+            if os.path.isfile(filename):
+                with open(filename) as fp:
+                    histogram_json = json.load(fp)
+                    return Histogram.from_json(histogram_json, verbose=verbose)
+            else:
+                LOGGER.error("Histogram.from_json: no such file %r", filename)
+                return
+        elif "histograms" in data:
+            retval = []
+            if verbose:
+                print("Histogram 3D:")
+            for datum in data["histograms"]:
+                histogram_json = datum["histogram"]
+                if verbose:
+                    print("Step:", datum["step"])
+                histogram = Histogram.from_json(histogram_json, verbose=verbose)
+                retval.append(histogram)
+            return retval
+        else:
+            histogram = Histogram(
+                data["start"], data["stop"], data["step"], data["offset"]
+            )
+            for (i, count) in data["index_values"]:
+                histogram.counts[i] = count
+            if verbose:
+                histogram.display()
+            return histogram
+
     def to_json(self):
         """
         Return histogram as JSON-like dict.
@@ -1100,6 +1236,14 @@ class Histogram(object):
             "step": self.step,
         }
 
+    def is_empty(self):
+        # type: () -> bool
+        """ Check if the Histogram is empty. Return True if empty, False otherwise
+        """
+        # If the Histogram contains at least one value, at least one element of
+        # self.counts will be not null
+        return not any(self.counts)
+
     def create_bin_values(self):
         """
         Create exponentially distributed bin values
@@ -1217,14 +1361,16 @@ class Histogram(object):
 
         return counts
 
-    def display(self, start, stop, step, format="%14.4f", show_empty=False):
+    def display(
+        self, start=None, stop=None, step=None, format="%14.4f", show_empty=False
+    ):
         """
         Show counts between start and stop by step increments.
 
         Args:
-            start: float, start of range to display
-            stop: float, end of range to display
-            step: float, amount to increment each range
+            start: optional, float, start of range to display
+            stop: optional, float, end of range to display
+            step: optional, float, amount to increment each range
             format: str (optional), format of numbers
             show_empty: bool (optional), if True, show all
                 entries in range
@@ -1237,6 +1383,7 @@ class Histogram(object):
         >>> history = Histogram()
         >>> values = [random.random() for x in range(10000)]
         >>> history.add(values)
+        >>> history.display()
 
         Histogram
         =========
@@ -1255,11 +1402,7 @@ class Histogram(object):
         -----------------------------------------------------------
         Total:     10000.0000
         """
-        counts = self.get_counts(start, stop, step)
-        current = start
-        total = 0.0
-        next_one = current + step
-        i = 0
+        collection = self.collect(start, stop, step)
         print("Histogram")
         print("=========")
         size = len(format % 0)
@@ -1268,23 +1411,69 @@ class Histogram(object):
         formats = [sformat % s for s in columns]
         print(*formats)
         print("-" * (size * 4 + 3))
-        while next_one <= stop + (step) and i < len(counts):
-            count = counts[i]
+        total = 0.0
+        for row in collection:
+            count = row["count"]
             total += count
             if show_empty or count > 0:
-                start_bin = self.get_bin_index(current)
-                stop_bin = self.get_bin_index(next_one)
                 print(
-                    format % current,
-                    format % next_one,
+                    format % row["value_start"],
+                    format % row["value_stop"],
                     format % count,
-                    (sformat % ("[%s-%s]" % (start_bin, stop_bin))),
+                    (
+                        sformat
+                        % ("[%s-%s]" % (row["bin_index_start"], row["bin_index_stop"]))
+                    ),
                 )
+        print("-" * (size * 4 + 3))
+        print(("Total: " + format) % total)
+
+    def collect(self, start=None, stop=None, step=None):
+        """
+        Collect the counts for the given range and step.
+
+        Args:
+            start: optional, float, start of range to display
+            stop: optional, float, end of range to display
+            step: optional, float, amount to increment each range
+
+        Returns a list of dicts containing details on each
+        virtual bin.
+        """
+        counts_compressed = self.counts_compressed()
+        if start is None:
+            if len(counts_compressed) > 0:
+                start = self.values[counts_compressed[0][0]]
+            else:
+                start = -1.0
+        if stop is None:
+            if len(counts_compressed) > 1:
+                stop = self.values[counts_compressed[-1][0]]
+            else:
+                stop = 1.0
+        if step is None:
+            step = (stop - start) / 10.0
+
+        counts = self.get_counts(start, stop + step, step)
+        current = start
+        bins = []
+        next_one = current + step
+        i = 0
+        while next_one <= stop + (step) and i < len(counts):
+            start_bin = self.get_bin_index(current)
+            stop_bin = self.get_bin_index(next_one)
+            bin = {
+                "value_start": current,
+                "value_stop": next_one,
+                "bin_index_start": start_bin,
+                "bin_index_stop": stop_bin,
+                "count": counts[i],
+            }
+            bins.append(bin)
             current = next_one
             next_one = current + step
             i += 1
-        print("-" * (size * 4 + 3))
-        print(("Total: " + format) % total)
+        return bins
 
 
 def write_numpy_array_as_wav(numpy_array, sample_rate, file_object):
@@ -1541,6 +1730,7 @@ def makedirs(name, exist_ok=False):
 
 
 def clean_and_check_root_relative_path(root, relative_path):
+    # type: (str, str) -> str
     """
     Given a root and a relative path, resolve the relative path to get an
     absolute path and make sure the resolved path is a child to root. Cases
@@ -1553,6 +1743,9 @@ def clean_and_check_root_relative_path(root, relative_path):
     if not os.path.isabs(root):
         raise ValueError("Root parameter %r should an absolute path" % root)
 
+    if not root.endswith(os.sep):
+        root = root + os.sep
+
     joined_path = os.path.join(root, relative_path)
     resolved_path = os.path.realpath(joined_path)
 
@@ -1562,6 +1755,17 @@ def clean_and_check_root_relative_path(root, relative_path):
     return resolved_path
 
 
+def check_if_path_relative_to_root(root, absolute_path):
+    # type: (str, str) -> bool
+    if not os.path.isabs(root):
+        raise ValueError("Root parameter %r should an absolute path" % root)
+
+    root_full_path = os.path.realpath(root) + os.sep
+    full_path = os.path.realpath(absolute_path)
+
+    return full_path.startswith(root_full_path)
+
+
 def verify_data_structure(datatype, data):
     # Raise an error if anything wrong
     if datatype == "curve":
